@@ -188,16 +188,22 @@ public class DescriptorParserImpl implements DescriptorParser {
     return parsedDescriptors;
   }
 
-  private static Descriptor parseDescriptor(byte[] rawDescriptorBytes,
+  static Descriptor parseDescriptor(byte[] rawDescriptorBytes,
       int[] offsetAndLength, Constructor<? extends DescriptorImpl> constructor,
       boolean failUnrecognizedDescriptorLines)
       throws DescriptorParseException {
     try {
       return constructor.newInstance(rawDescriptorBytes,
           offsetAndLength, failUnrecognizedDescriptorLines);
-    } catch (InstantiationException | IllegalAccessException
-        | InvocationTargetException e) {
-      throw new RuntimeException();
+    } catch (InvocationTargetException e) {
+      if (null != e.getCause()
+          && e.getCause() instanceof DescriptorParseException) {
+        throw (DescriptorParseException) e.getCause();
+      } else {
+        throw new RuntimeException(e);
+      }
+    } catch (InstantiationException | IllegalAccessException e) {
+      throw new RuntimeException(e);
     }
   }
 
